/***************************************************************************************** 
*                                     TinyRtos51 
*                                The Real-Time Kernel 
*                              Copyright(c) 2024 fanxsp
*                         SPDX-License-Identifier: APACHE-2.0
*       This software is subject to an open source license and is distributed by fanxsp,
*       pursuant to the terms of the Apache License.
******************************************************************************************/
/******************************************************************************************* 
* 				main.c文件
* Author: fanxsp 
* Revision: 1.00
* Date: 2023-05-03
* 	1. 在tinyrtos51cfg.h 中配置 OS
*   2. 在port.h port_xxx.h中适配不同型号的 MCU
*   3. 在option的c51中的预定义宏中定义目标MCU型号
*   4. 如果使能定时服务，需要在一个定时器中调用OSTimeTick()函数，定时中断应设为最低优先级
********************************************************************************************/

/*************  功能说明    ****************************************************************
* 本例程基于STC8H8K64U核心实验板（开天斧）进行编写测试。
* 用逻辑分析仪测试任务切换时间，串口波特率9600 @ 时钟 22.1184M
********************************************************************************************/
#include "tinyRtos51.h"

#define MAIN_Fosc       22118400L   
#define Timer0_Reload   (65536UL -(MAIN_Fosc / 1000))       //Timer0 中断频率 1000次/秒

/* 任务定义 -------------------------------------------------------------------------------*/
/* 任务堆栈  */
OStypeInt8u OSLOC_STACKS TFLedStack[30] ;
OStypeInt8u OSLOC_STACKS TFKeyStack[30] ;
/* 任务ID  */
#define ID_TFLed     0
#define ID_TFKey     1
/* 任务优先级 */
#define PRIO_TFLed   4
#define PRIO_TFKey   5
/* 任务函数原型 */
void TFLed(void) ;
void TFKey(void) ;
/*-----------------------------------------------------------------------------------------*/

void Init(void) ;
void Uart1_Init(void) ;

void main(void)
{
/* OS初始化  */
	OSInit() ;
	
/* 创建任务、二值信号量、计数信号量、事件标志、消息队列  放在这里 */
	OSCreateTask(ID_TFLed,TFLed,TFLedStack,PRIO_TFLed) ;
	OSCreateTask(ID_TFKey,TFKey,TFKeyStack,PRIO_TFKey) ;
	
/*----------------------------------------------------------------------------------------*/
	
/* 用户初始化 放在这里  */
	Init() ;
/*----------------------------------------------------------------------------------------*/
	
/* 启动OS */ 
	OSStart() ;
	
/* 后面不要放任何代码 */
	
}

/* LED控制任务  */
void TFLed(void)
{
	P2 = 0 ;
	for(;;)
	{
		OSTaskSuspend() ;
		P21 = 1 ;
		OSDelay(2) ;
		P20 = 0 ;
		P21 = 0 ;
	}
}

/* 按键检测 任务 */
void TFKey(void)
{
	static unsigned char Key1Cnt2,Key1Cnt3,Key1Cnt4,Key1Cnt5 ;
	
	for(;;)
	{
		if(!P32)
		{
			if(Key1Cnt2 < 0xff) Key1Cnt2 ++ ;
		}
		else
		{
			Key1Cnt2 = 0 ;
		}
		
		if(!P33)
		{
			if(Key1Cnt3 < 0xff) Key1Cnt3 ++ ;
		}
		else
		{
			Key1Cnt3 = 0 ;
		}
		
		if(!P34)
		{
			if(Key1Cnt4 < 0xff) Key1Cnt4 ++ ;
		}
		else
		{
			Key1Cnt4 = 0 ;
		}
		
		if(!P35)
		{
			if(Key1Cnt5 < 0xff) Key1Cnt5 ++ ;
		}
		else
		{
			Key1Cnt5 = 0 ;
		}
		
		if(Key1Cnt2 == 2)
		{
			P20 = 1 ;
			OSTaskResume(ID_TFLed) ;
		}
		
		if(Key1Cnt3 == 2)
		{

		}
		
		if(Key1Cnt4 == 2)
		{

		}
		
		if(Key1Cnt5 == 2)
		{

		}
		
		OSDelay(1) ;
	}
}

void ISR_Timer0(void) interrupt 1  // 1ms 中断函数
{
	OSTimeTick() ;    // OS节拍处理
}

void ISR_Uart1(void) interrupt 4  // UART1 中断
{
	if(TI)
	{
		TI = 0 ;

	}
	
	if(RI)
	{
		RI = 0 ;
		P20 = 1 ;
		OSIntResume(ID_TFLed) ;
	}
}

void Init(void)
{
	P_SW2 |= 0x80;                //扩展寄存器(XFR)访问使能
	P0M1 = 0x00;   P0M0 = 0x00;   //设置为准双向口
	P1M1 = 0x00;   P1M0 = 0x00;   //设置为准双向口
	P2M1 = 0x00;   P2M0 = 0x00;   //设置为准双向口
	P3M1 = 0x00;   P3M0 = 0x00;   //设置为准双向口
	P4M1 = 0x00;   P4M0 = 0x00;   //设置为准双向口
	P5M1 = 0x00;   P5M0 = 0x00;   //设置为准双向口
	P6M1 = 0x00;   P6M0 = 0x00;   //设置为准双向口
	P7M1 = 0x00;   P7M0 = 0x00;   //设置为准双向口
	
	Uart1_Init() ;
	
	AUXR |= 0x80;   //Timer0 1T, 16 bits 自动重装载模式 
	TH0 = (unsigned char)(Timer0_Reload / 256);
	TL0 = (unsigned char)(Timer0_Reload % 256);
	ET0 = 1;        //Timer0 中断使能
	TR0 = 1;        //Tiner0 开
}

void Uart1_Init(void)	//9600bps@22.1184MHz
{
	SCON = 0x50;		//8位数据,可变波特率
	AUXR |= 0x01;		//串口1选择定时器2为波特率发生器
	AUXR |= 0x04;		//定时器时钟1T模式
	T2L = 0xC0;			//设置定时初始值
	T2H = 0xFD;			//设置定时初始值
	AUXR |= 0x10;		//定时器2开始计时
	ES = 1 ;            //使能中断
	PS = 1 ;
	IPH &= ~0x10 ;      // 优先级 1
	AUXR |=  (1<<4);
	AUXR &= ~(1<<1) ;
}

