/***************************************************************************************** 
*                                     TinyRtos51 
*                                The Real-Time Kernel 
*                              Copyright(c) 2024 fanxsp
*                         SPDX-License-Identifier: APACHE-2.0
*       This software is subject to an open source license and is distributed by fanxsp,
*       pursuant to the terms of the Apache License.
******************************************************************************************/
/******************************************************************************************* 
* 				main.c文件
* Author: fanxsp 
* Revision: 1.00
* Date: 2023-05-03
* 	1. 在tinyrtos51cfg.h 中配置 OS
*   2. 在port.h port_xxx.h中适配不同型号的 MCU
*   3. 在option的c51中的预定义宏中定义目标MCU型号
*   4. 如果使能定时服务，需要在一个定时器中调用OSTimeTick()函数，定时中断应设为最低优先级
********************************************************************************************/

/*************  功能说明    ****************************************************************
* 本例程基于STC8H8K64U核心实验板（开天斧）进行编写测试。
* 测试目的：reentrant 函数使用,通过仿真观察模拟软堆栈的保存、恢复及切换,
*           通过led观察func1()中d的数值
********************************************************************************************/
#include "tinyRtos51.h"

#define MAIN_Fosc       24000000L   
#define Timer0_Reload   (65536UL -(MAIN_Fosc / 1000))       //Timer0 中断频率 1000次/秒

/* 任务定义 -------------------------------------------------------------------------------*/
/* 任务堆栈  */
OStypeInt8u OSLOC_STACKS taskStack0[40] ;
OStypeInt8u OSLOC_STACKS taskStack1[40] ;
/* 任务ID  */
#define ID_TASK0  0
#define ID_TASK1  1
/* 任务优先级 */
#define PRIO_TASK0  5
#define PRIO_TASK1  6
/* 任务函数原型 */
void Task0(void) ;
void Task1(void) ;
/*-----------------------------------------------------------------------------------------*/

void Init(void) ;
//void func1(unsigned char m, unsigned char a,unsigned char b,unsigned char c) large reentrant ;
void func1(unsigned char m, unsigned char a,unsigned char b,unsigned char c) small reentrant ;

void main(void)
{
/* OS初始化  */
	OSInit() ;
	
/* 创建任务、二值信号量、计数信号量、事件标志、消息队列  放在这里 */
	OSCreateTask(ID_TASK0,Task0,taskStack0,40,5,PRIO_TASK0) ;
	OSCreateTask(ID_TASK1,Task1,taskStack1,40,5,PRIO_TASK1) ;

/*----------------------------------------------------------------------------------------*/
	
/* 用户初始化 放在这里  */
	Init() ;
/*----------------------------------------------------------------------------------------*/
	
/* 启动OS */ 
	OSStart() ;
	
/* 后面不要放任何代码 */
}

/* 任务0  */
void Task0(void)
{
	for(;;)
	{
		func1(0,0,1,2) ;
	}
}

/* 任务1 */
void Task1(void)
{
	for(;;)
	{
		func1(1,1,2,3) ;
	}
}

#if OSENABLE_REENTRANT_LARGE > 0
#pragma disable
#endif
void ISR_Timer0(void) interrupt 1  // 1ms 中断函数
{
#if OSENABLE_REENTRANT_LARGE > 0
	OSEnterInt() ;
#endif	
	func1(2,2,3,4) ;
	_nop_() ;
	OSTimeTick() ;    // OS节拍处理
#if OSENABLE_REENTRANT_LARGE > 0
	OSExitInt() ;
#endif
}

void Init(void)
{
	P_SW2 |= 0x80;     //扩展寄存器(XFR)访问使能
	P0M1 = 0x00;   P0M0 = 0x00;   //设置为准双向口
	P1M1 = 0x00;   P1M0 = 0x00;   //设置为准双向口
	P2M1 = 0x00;   P2M0 = 0x00;   //设置为准双向口
	P3M1 = 0x00;   P3M0 = 0x00;   //设置为准双向口
	P4M1 = 0x00;   P4M0 = 0x00;   //设置为准双向口
	P5M1 = 0x00;   P5M0 = 0x00;   //设置为准双向口
	P6M1 = 0x00;   P6M0 = 0x00;   //设置为准双向口
	P7M1 = 0x00;   P7M0 = 0x00;   //设置为准双向口
	
	AUXR = 0x80;    //Timer0 1T, 16 bits 自动重装载模式 
	TH0 = (unsigned char)(Timer0_Reload / 256);
	TL0 = (unsigned char)(Timer0_Reload % 256);
	ET0 = 1;    //Timer0 中断使能
	TR0 = 1;    //Tiner0 开
}

//void func1(unsigned char m, unsigned char a,unsigned char b,unsigned char c) large reentrant
void func1(unsigned char m, unsigned char a,unsigned char b,unsigned char c) small reentrant
{
	unsigned char d ;
	
	if(m == 0)
	{
		OSTaskSuspend() ;
		_nop_() ;
	}
	else if(m == 1)
	{
		OSTaskResume(ID_TASK0) ;
		_nop_() ;
	}
	
	d = a + b + c ;
	P2 = ~d ;
}

